#!/usr/bin/env python

from distutils.core import setup

setup(name='reparted',
      version='1.1',
      description='Python libparted bindings',
      author='Ruben Quinones',
      author_email='rq.sysadmin@gmail.com',
      url='http://github.com/xzased/reparted',
      packages=['reparted'],
     )